/**************************************************
 ******           Liss-Etymino V1.0          ******
 ******   Verallgemeinerte Lissajous-Kurven  ******
 **************************************************
 ******     Progamm zum Roman "Curiepolis"   ******
 ******          von Fabian Herrmann         ******
 ******        geocities.ws/curiepolis       ******
 **************************************************
 ******            3. August 2017            ******
 **************************************************
 
 
 
******************************************************************************************

  Schildkröten, handtellergroße mit schwarzem, gelbegränderten Panzer und winzigen 
  schaufelförmigen Köpfen, ruderten geschäftig, unaufhörlich mit ihren vier Stummel-
  beinen arbeitend, durch die Tiefe, kurvten und tauchten und tummelten sich um-
  einander, Dutzende, vielleicht Hunderte mussten es sein, die in dem kleinen 
  Gewässer lebten, in einer begrenzten Welt voller Schlamm und Wärme und nahrhafter 
  Schwebstoffe, Würmer, Insektenlarven, Frösche, Schnecken, kieselzarte Süß-
  wassermuscheln. Gelegentlich kam eine von ihnen an die Oberfläche, die sie mit 
  ihrer winzigen grauen Nase durchstieß um zu atmen, dabei entstand kaum eine Welle, 
  kaum wurde das Wasser aufgewühlt, die Oberfläche war glatt und schwarz wie die 
  eines Ölteichs. Das Gekraule und Gewirbel der Schildkröten, ihr unermüdliches 
  Rudern Tauchen Gleiten, war lustig zu sehen; Annika fühlte sich zufrieden und 
  froh, und sie lauschte: Stimmen, eine Art dunkle, pulsierende, erdige Musik klang 
  herauf – die Schildkröten sangen. Sie sangen ein glückliches Lied vom Leben im 
  düsterwarmen Wasser, ein sorgloses Schildkrötenlied von ihrem Dasein in der Tiefe 
  des Teiches, den sie als ganze Welt ansahen, jenseits des Wassers gab es den 
  erfrischenden Luftozean, in den sie von Zeit zu Zeit ihre Nasen stießen, den 
  hielten sie für die äußerste Grenze des Universums. Ein geschäftiges, fröhliches, 
  aber begrenztes Leben, das sie mit ihrem ewigen Gesang feierten. Doch sie hatten 
  Annika bemerkt, ein außerordentliches, schönes, göttergleiches Wesen aus einem 
  anderen Universum.

******************************************************************************************/


/* Schildkröten-Geometrie nach Seymour Papert. Die Klasse kann als "Black Box" benutzt werden,
   d.h. die Leserin kann sie einsetzen, ohne die internen Berechnungen verstehen zu müssen.
   (Letztere sind allerdings auch nicht besonders kompliziert -- elementare Trigonometrie. */
   
class Turtle
{
  // Interne Variablen
  float xleft, ytop, xright, ybottom, whalf, hhalf;
  
  double xT, yT, hT;
  double xscale, yscale;
  
  double r2d;
  
  boolean pen;
  
  color cT;
  
  int Graph;
  
  
  // Initialisierungsfunktion. 
  // Parameter: 
  //    xl, yt, xr, yb = linke-obere und rechte-untere Ecke des Schildkrötenzeichenfeldes (gemessen in Monitorkoordinaten).
  //    xsc, ysc       = Skalierungsfaktor zwischen Schildkröten- und tatsächlicher Zeichenebene auf dem Monitor. (Meist ist 1.0 eine gute Wahl.)
  //    g              = Nummer des Grafikkontextes. (Siehe unten, vor setup()-Funktion.)
  Turtle(float xl, float yt, float xr, float yb, double xsc, double ysc, int g)
  {
    xleft = xl;
    ytop = yt;
    xright = xr;
    ybottom = yb;
    xscale = xsc;
    yscale = ysc;
    
    whalf = (xright-xleft)/2;
    hhalf = (ybottom-ytop)/2;
    
    r2d = PI/180.0;
    
    xT = 0.0;
    yT = 0.0;
    hT = 0.0;
    
    pen = true;
    
    cT = color(0,0,0);
    
    Graph = g;
  }
  
  // Vorwärts
  void fd(double l)
  {
    double altx = xT;
    double alty = yT;
    
    xT += l*sin((float)hT);
    yT += l*cos((float)hT);
    if(pen) tdraw(altx, alty, xT, yT);
  }
  
  // Rückwärts
  void bk(double l)
  {
    fd(-l);
  }
  
  // Rechtsdrehen (in Grad)
  void rt(double w)
  {
    hT += w*r2d;
  }
  
  // Linksdrehen (in Grad)
  void lt(double w)
  {
    hT -= w*r2d;
  }
  
  // Richtung h (in Grad; 0 oben, 90 rechts, 180 unten, etc.) anpeilen
  void seth(double h)
  {
    hT = h*r2d;
  }
  
  // Auf Position x, y springen
  void setpos(double x, double y)
  {
    double altx = xT;
    double alty = yT;
    
    xT = x;
    yT = y;
    
    if(pen) tdraw(altx, alty, xT, yT);
  }
  
  // Ab nach Hause (Koordinaten 0, 0)
  void home()
  {
    setpos(0, 0);
    seth(0);
  }
  
  // Stift hoch
  void pu()
  {
    pen = false;
  }
  
  // Stift runter
  void pd()
  {
    pen = true;
  }
  
  // Stiftfarbe wählen (Farbwerte 0...255 einzeln)
  void setc(float r, float g, float b)
  {
    cT = color(r, g, b);
  }
  
  // Stiftfarbe wählen (hexadezimal)
  void setC(color c)
  {
    cT = c;
  }
  
  // Punkt x, y anpeilen
  void towards(double x, double y)
  {
    double dx = x-xT;
    double dy = y-yT;
    double dr = sqrt((float)(dx*dx+dy*dy));
    if(dr < 1e-3) return;
    
    double h = asin((float)(dx/dr));
    
    if(dx>=0 && dy>=0)
    {
      hT = h;
    }
    else if(dx>=0 && dy<0)
    {
      hT = PI-h;
    }
    else if(dx<0 && dy<0)
    {
      hT = PI-h;
    }
    else
    {
      hT = 2*PI+h;
    }
  }
  
  // Schildkrötenkurs zurückgeben (in Grad)
  double head()
  {
    return hT/r2d;
  }
  
  // Interne Funktion: Ziehen der Linie
  void tdraw(double x1, double y1, double x2, double y2)
  {
    float xa, ya, xe, ye;
    
    xa = xleft   + whalf + (float)(xscale*x1);
    xe = xleft   + whalf + (float)(xscale*x2);
    ya = ytop    + hhalf - (float)(yscale*y1);
    ye = ytop    + hhalf - (float)(yscale*y2);
    
    myline((int)xa, (int)ya, (int)xe, (int)ye, cT, Graph);
  }
  
  // Schildkröten-x-Koordinate auf realem Zeichenfeld zurückgeben
  float xt2s()
  {
    return xleft   + whalf + (float)(xscale*xT);
  }
  
  // wie oben, y-Koordinate
  float yt2s()
  {
    return ytop    + hhalf - (float)(yscale*yT);
  }
  
  
  // PS. Wer als Kind nicht in LOGO programmiert hat, sieht aus wie Ronny Heßling.
}


// Eigene Linienfunktion mit Integer-Bresenham-Algorithmus
// Parameter: Start-/Zielpunkt, Farbe, Nr. des Grafikkontextes
void myline(int x0, int y0, int x1, int y1, color c, int g)
{
  if(x0==x1) 
  {
    vline(x0, y0, y1, c, g);
    return;
  }
  
  if(y0==y1) 
  {
    hline(x0, x1, y0, c, g);
    return;
  }
  
  double deltax = (double)(x1 - x0);
  double deltay = (double)(y1 - y0);
  double deltaerr = deltay / deltax;    // Assume deltax != 0 (line is not vertical)
  if(deltaerr<0) deltaerr = -deltaerr;
  double error = deltaerr - 0.5;
  
  if(deltaerr<1)
  {
    int y = y0;
  
    for(int x=x0; x!=x1; x+=(x0<x1)?1:-1)
    {
           setzepix(x,y,c,g);
           error = error + deltaerr;
           if(error >= 0.5)
           {
               y+=(y0<y1)?1:-1;
               error = error - 1.0;
           }
    }
  
  }
  else
  {
    deltaerr = 1.0/deltaerr;
    error = deltaerr - 0.5;
    int x = x0;
  
    for(int y=y0; y!=y1; y+=(y0<y1)?1:-1)
    {
         setzepix(x,y,c,g);
         error = error + deltaerr;
         if(error >= 0.5)
         {
             x+=(x0<x1)?1:-1;
             error = error - 1.0;
         }
    }
  
  }
}

// Hilfsfunktion für vertikale Linien
void vline(int x, int y0, int y1, color c, int g)
{
  for(int y=y0; y!=y1; y+=(y0<y1)?1:-1)
  {
     setzepix(x,y,c,g);
  }
}

// Ebenso; für horizontale Linien
void hline(int x0, int x1, int y, color c, int g)
{
  for(int x=x0; x!=x1; x+=(x0<x1)?1:-1)
  {
     setzepix(x,y,c,g);
  }
}

// Hilfsfunktion: Setzt einen einzelnen Pixel
void setzepix(int x, int y, color c, int g)
{
  if(x>width-1 || x<0 || y>height-1 || y<0) return;
  int i = y*G[g].width+x;
  
  G[g].pixels[i] = c;
}




/**************** Hauptprogramm ****************/


// Feld für die Grafikkontexte. Es ist sinnvoll, mit Kontexten
// zu arbeiten, da man diese wie die Ebenen in GIMP o.ä. 
// kombinieren und einschließlich Transparenzwerten abspeichern kann.
PGraphics[] G;

// Schildkröte
Turtle A;


/*********************************************************************************
 
 Eswarneunzehnhundertdreiundneunzig&&wirwusstennichwohin-Klacktatur mit Spiralkabel – 
 stahlblauer Märzmorgen in Cyberia: Große Sehnsucht Hoffnung Freiheit – Zwotastmaus & 
 Zehärrtee in sechshundertvierzig-icks-vierhundertachtzig-icks-sechzehn: Farbpalette 
 passend bemessen für'n hübsches Mädchen in ebendiesem Alter.

**********************************************************************************/

color cga[] = new color[16];


// Klasse, die Parameter für die verallgemeinerten Lissajouskurven enthält.
// Wir legen später Felder dieses Datentyps an, jeder Feldeintrag entspricht
// einem "Federpaar" des Lissajous-Mechanismus'. Siehe: http://geocities.ws/curiepolis/zeitung#2-8-17
class lissdata
{
  double Ax, Ay, fx, fy, delta;
  
  // Initialisator.
  // Parameter:
  //     Axx, Ayy = Amplitude der ersten und zweiten Feder
  //     fxx, fyy = Frequenz   ~    ~     ~     ~      ~
  //       delta1 = Phasenverschiebung zwischen ihnen
  lissdata(double Axx, double Ayy, double fxx, double fyy, double delta1)
  {
    Ax = Axx;
    Ay = Ayy;
    fx = fxx;
    fy = fyy;
    delta = delta1;
  }
}



// Programmvorbereitung
void setup()
{
  // Vollbild, schwarzer Hintergrund, kein Schleifendurchlauf
  fullScreen();
  background(0);
  noLoop();
  
  // Zehn Blautöne erzeugen
  int j = 0;
  for(float a=0.1; a<1.05; a+=0.1, j++)
  {
    cga[j] = lerpColor(color(0), color(0, 0, 255), a);
  }
  // Weitere Farben
  cga[11] = #aa0000;
  cga[12] = #00aa00;
  cga[13] = #ccaa00;
  cga[14] = #aa00aa;
  cga[15] = #ffffff;
  
  // Feld von Grafikkontexten erzeugen
  G = new PGraphics[10];
  
  // Grafikkontexte initialisieren und zeichenbereit machen
  for(int i=0; i<10; i++)
  {
    G[i] = createGraphics(width, height);
    G[i].beginDraw();
  }
  
  // Turtle erzeugen
  A = new Turtle(0, 0, width, height, 1.0, 1.0, 0);
}


// Himmelsstreifen zeichnen
void sky()
{
  int i;
  noStroke();
  for(i=0; i<10; i++)
  {
    fill(cga[i]);
    rect(0, i*height/10, width, (i+1)*height/10);
  }
}



/********************************************************************************************

 Vertrautestes der Längeren Gedankenspiele: Woanders-sein. Himmelsblick der All-1en. 
 (Mantelärmel in Mantelärmel? Niemand hat zwei rechte Hände.) Woanders: Blick-in-Blick 
 – Energieflussdichte der Seele – vielleicht erzählt man sich was && kichert universono... 
 double Ax = 17.5, Ay = 20.0, delta = 0.0, r1, r2; for(double theta = 0; Ax < 1000; 
 Ax *= 1.00001, Ay *= 1.00001, theta += 0.002) {r1 = Ax * sin(theta); r2 = Ay * cos(fy*theta 
 + delta); fy += 3.0e-8; delta += 3.0e-6;}  (D.h.: n Tick ovallänglich, Talschüsselchen 
 schattigtief, draus blumt'ΔS hervor; verborgen unter Bluse Pullover Mantel – Zeit und Ort 
 und Wind bedeutungslos.)

 ********************************************************************************************/


// Hauptroutine, malt das eigentliche Bild
void draw()
{
  // Gescanntes Aquarell von Annika laden...
  PImage annika = loadImage("nude2.png");
  // ...und in Grafikkontext 2 einfügen.
  G[2].image(annika, width*0.5, height*0.1);
  G[2].endDraw();
  
  // Erstes Etymino-Bild in Grafikkontext 0
  // (Je nachdem, welcher Funktionsaufruf auskommentiert ist oder nicht, entsteht ein anderes Bild.)
  G[0].loadPixels();
  //mannig(0, 11);
  feldlinien(0, 11);
  G[0].updatePixels();
  
  // Zweites Etymino-Bild in Grafikkontext 1
  G[1].loadPixels();
  //zeitortwind(1, 14, 0, 0); //-width/2 + annika.width*0.45, height/2 - annika.height*0.44);
  galaxis(1, 14);
  G[1].updatePixels();
  

  // Kombinieren von Grafikkontext 0 und 1 mit der Funktion outline2(), Ergebnis in Kontext 8.
  // (Oder einer anderen -- siehe auskommentierte Zeilen.)
  //spray(0, 1, 2, 9, 25, 5);
  outline2(0, 1, 8, 13);
  //combine(8, 2, 9, 9);
  
  // Himmel zeichnen.
  sky();
  
  // Kontext 8 zu Bildschirm, darüber Annika in Kontext 2.
  image(G[8], 0, 0);
  image(G[2], 0, 0);
  //image(G[1], 0, 0);
  
  // Bild speichern.
  // (Auskommentieren, wenn nicht gespeichert werden soll.)
  save("Annika-galaxy.png");
}


// Pixel-Kombinationsroutinen, die Inhalte verschiedener Grafikkontexte verknüpfen.
// Parameter:
//     g1, g2 = die beiden Quellkontexte
//     g3     = Zielkontext
//     c      = Kombinationsfarbe (int-Wert, entsprechend einem Eintrag im cga[]-Feld)

// Ineinanderkopieren; dort, wo zwei Pixel zusammenfallen, die Farbe in c ändern.
void combine(int g1, int g2, int g3, int c)
{
  G[g1].loadPixels();
  G[g2].loadPixels();
  G[g3].loadPixels();
  for(int i=0; i<width*height; i++)
  {
    if(alpha(G[g1].pixels[i])==255 && alpha(G[g2].pixels[i])>0)
    {
      G[g3].pixels[i] = cga[c];
    }
    else if(alpha(G[g1].pixels[i])>0)
    {
      G[g3].pixels[i] = G[g1].pixels[i];
    }
    else if(alpha(G[g2].pixels[i])>0)
    {
      G[g3].pixels[i] = G[g2].pixels[i];
    }
  }
  
  G[g3].updatePixels();
}

// Wie combine(), dabei werden die Pixel jedoch aufgefächert und vervierfacht
void outline(int g1, int g2, int g3, int c)
{
  G[g1].loadPixels();
  G[g2].loadPixels();
  G[g3].loadPixels();
  for(int i=0; i<width*height; i++)
  {
    if(alpha(G[g1].pixels[i])==255 && alpha(G[g2].pixels[i])==255)
    {
      G[g3].pixels[i] = color(0, 0, 0, 0);
      if(i>5) G[g3].pixels[i-5] = cga[c];
      if(i<width*height-6) G[g3].pixels[i+5] = cga[c];
      if(i>=width*5) G[g3].pixels[i-width*5] = cga[c];
      if(i<width*height-width*5-1) G[g3].pixels[i+width*5] = cga[c];
    }
    else if(alpha(G[g1].pixels[i])==255)
    {
      if(i>5) G[g3].pixels[i-5] = G[g1].pixels[i];
      if(i<width*height-6) G[g3].pixels[i+5] = G[g1].pixels[i];
      if(i>=width*5) G[g3].pixels[i-width*5] = G[g1].pixels[i];
      if(i<width*height-width*5-1) G[g3].pixels[i+width*5] = G[g1].pixels[i];
    }
    else if(alpha(G[g2].pixels[i])==255)
    {
      if(i>5) G[g3].pixels[i-5] = G[g2].pixels[i];
      if(i<width*height-6) G[g3].pixels[i+5] = G[g2].pixels[i];
      if(i>=width*5) G[g3].pixels[i-width*5] = G[g2].pixels[i];
      if(i<width*height-width*5-1) G[g3].pixels[i+width*5] = G[g2].pixels[i];
    }
  }
  
  G[g3].updatePixels();
}


// Entspricht outline(), aber mit verbessertem Algorithmus, der keinen "Überhang" zwischen
// linkem und rechtem Bildrand erzeugt.
void outline2(int g1, int g2, int g3, int c)
{
  G[g1].loadPixels();
  G[g2].loadPixels();
  G[g3].loadPixels();
  for(int x=0; x<width; x++)
  {
    for(int y=0; y<height; y++)
    {
      int i = y*width+x;
      if(alpha(G[g1].pixels[i])==255 && alpha(G[g2].pixels[i])==255)
      {
        G[g3].pixels[i] = color(0, 0, 0, 0);
        if(x>5) G[g3].pixels[i-5] = cga[c];
        if(x<width-6) G[g3].pixels[i+5] = cga[c];
        if(y>5) G[g3].pixels[i-width*5] = cga[c];
        if(y<height-6) G[g3].pixels[i+width*5] = cga[c];
      }
      else if(alpha(G[g1].pixels[i])==255)
      {
        if(x>5) G[g3].pixels[i-5] = G[g1].pixels[i];
        if(x<width-6) G[g3].pixels[i+5] = G[g1].pixels[i];
        if(y>5) G[g3].pixels[i-width*5] = G[g1].pixels[i];
        if(y<height-6) G[g3].pixels[i+width*5] = G[g1].pixels[i];
      }
      else if(alpha(G[g2].pixels[i])==255)
      {
        if(x>5) G[g3].pixels[i-5] = G[g2].pixels[i];
        if(x<width-6) G[g3].pixels[i+5] = G[g2].pixels[i];
        if(y>5) G[g3].pixels[i-width*5] = G[g2].pixels[i];
        if(y<height-6) G[g3].pixels[i+width*5] = G[g2].pixels[i];
      }
    }
  }
  
  G[g3].updatePixels();
}


// Entspricht combine(), erzeugt jedoch dort, wo zwei Pixel zusammenfallen, einen Sprayeffekt
// in Farbe c. N = Anzahl der Sprühpunkte, r = Sprühradius.
void spray(int g1, int g2, int g3, int c, int N, int r)
{
  G[g1].loadPixels();
  G[g2].loadPixels();
  G[g3].loadPixels();
  for(int x=0; x<width; x++)
  {
    for(int y=0; y<height; y++)
    {
      int i = y*width + x;
      if(alpha(G[g1].pixels[i])==255 && alpha(G[g2].pixels[i])==255)
      {
        
        for(int j=0; j<N; j++)
        {
          float w  = random(2*PI);
          float rr = random(r);
          float xx = (float)x + rr*cos(w);
          float yy = (float)y + rr*sin(w);
          int xxx  = (int)xx;
          int yyy  = (int)yy; 
          if(xxx>=0 && xxx<width && yyy>=0 && yyy<height)
          {
            G[g3].pixels[yyy*width + xxx] = cga[c];
          }
        }
      }
      else if(alpha(G[g1].pixels[i])==255)
      {
        G[g3].pixels[i] = G[g1].pixels[i];
      }
      else if(alpha(G[g2].pixels[i])==255)
      {
        G[g3].pixels[i] = G[g2].pixels[i];
      }
    }
  }
  
  G[g3].updatePixels();
}


// Malroutinen

// Parameter:
//    g = Nr. des Grafikkontextes (der vor Funktionsruf mit loadPixels() initialisiert,
//        danach mit updatePixels() aktualisiert werden sollte.
//    c = Nr. der Farbe (siehe cga[]-Feld).

void feldlinien(int g, int c)
{
  // Feld zur Übergabe der Parameter für die verallgemeinerten Lissajouskurven
  lissdata[] L;
  L = new lissdata[4];
  
  // Füttern des Feldes mit Parametern
  for(int i=0; i<4; i++)
  {
    float ii = (float)(i+1);
    L[i] = new lissdata(1.5/ii, 2/ii, ii*2, ii*3, ii*PI/6);
  }
  
  // Aufruf der eigentlichen Zeichenroutine
  liss1(4, 120, g, c, 1.0001, 1.0001, 0, 0, L, 0, 0);
}


// (die übrigen Malroutinen funktionieren analog.)

void mannig(int g, int c)
{
  lissdata[] L;
  L = new lissdata[4];
  
  for(int i=0; i<4; i++)
  {
    float ii = (float)(i+1);
    L[i] = new lissdata(1.5/ii, 2/ii, ii*2, ii*3, ii*PI/6);
  }
  
  L[3].Ax = 1.7;
  L[3].Ay = 3;
  L[3].fy = 2;
  
  liss1(4, 120, g, c, 1.00003, 1.00003, 0, 0, L, 0, 0);
}


void galaxis(int g, int c)
{
  lissdata[] L;
  L = new lissdata[4];
  
  for(int i=0; i<4; i++)
  {
    float ii = (float)(i+1);
    L[i] = new lissdata(17.5/ii, 20/ii, ii, ii, 0);
  }
  
  liss1(1, -90, g, c, 1.0001, 1.0001, 3e-8, 3e-6, L, 0, 0);
}


void zeitortwind(int g, int c, double xoff, double yoff)
{
  lissdata[] L;
  L = new lissdata[4];
  
  for(int i=0; i<4; i++)
  {
    float ii = (float)(i+1);
    L[i] = new lissdata(17.5/ii, 20/ii, ii, ii, 0);
  }
  
  liss1(1, -90, g, c, 1.00001, 1.00001, 3e-8, 3e-6, L, xoff, yoff);
}




// Zeichenroutine für verallgemeinerte Lissajouskurven

// In dieser Variante treten die Federn stets in Paaren auf, die Winkel zwischen ihnen haben einen konstanten Wert
// (90 Grad bei einem einzelnen Federpaar entspricht daher einer klassischen Lissajouskurve).

// Parameter:
//     n = Anzahl der Federpaare
//     w = Winkel zwischen den Federn (in Grad)
//     g = Nr. des Grafikkontextes
//     c = Nr. der Farbe (cga[]-Feld)
//     Axm, Aym   = Faktor, mit der die Federamplituden nach jedem Zeichenschritt multipliziert werden
//                  (jeweils die erste Feder jedes Paares mit Axm, die zweite mit Aym).
//     fp         = Wert, der zu den Frequenzen nach jedem Zeichenschritt addiert wird.
//     dp         = Ditto, für die Phasenverschiebungen
//     Liss       = Feldvariable mit Parametern (siehe oben)
//     xoff, yoff = x- und y-Offest, um den die Grafik verschoben wird (Turtlekoordinaten, also klassisches kartesisches Koordinatennetz).

void liss1(int n, double w, int g, int c, float Axm, float Aym, float fp, float dp, lissdata[] Liss, double xoff, double yoff)
{
  // Laufvariable für den Winkel, der in die trigonometrischen Funktionen eingeht
  float theta;
  // Startrichtung (nach rechts, bzw. "Osten)
  double w1 = 90;
  
  // Wir befestigen jeweils am Ende jedes Federpaares einen Zeichenstift
  double[] x0, y0, x1, y1;
  x0 = new double[n];
  y0 = new double[n];
  x1 = new double[n];
  y1 = new double[n];
  
  // Erster Schleifendurchlauf?
  boolean firstp = true;
  
  // Zeichenschleife, in der theta hochgezählt wird
  // Abbruch, wenn die Länge der ersten Feder die halbe Bildschirmbreite überschreitet
  for(theta = 0; Liss[0].Ax < width/2; theta += 0.002) //<>//
  {
    
    // Turtle-Stift hoch
    A.pu();
    // Auf Startposition
    A.setpos(xoff, yoff);;
    // Startrichtung einstellen
    A.seth(w1);
    
    // Durchlaufen der Federpaare
    for(int i=0; i<n; i++)
    {
      // Interne Variablen zum Zwischenspeichern von Werten
      double Ax, Ay, fx, fy, delta; 
      
      // Die Amplituden werden nocheinmal durch ihren Index+1 dividiert
      Ax = Liss[i].Ax / (double)(i+1);
      Ay = Liss[i].Ay / (double)(i+1);
      fx = Liss[i].fx;
      fy = Liss[i].fy;
      delta = Liss[i].delta;
      
      // Berechnen der Auslenkungen
      double r1, r2;
      r1 = Ax * sin((float)(fx*theta));
      r2 = Ay * cos((float)(fy*theta + delta));
      
      // Bewegen der Turtle
      A.fd(r1);
      A.lt(w);
      A.fd(r2);
      A.lt(w);
      
      // Merken der Koordinaten
      x0[i] = A.xt2s();
      y0[i] = A.yt2s();
      
      // Falls nicht erster Schleifendurchlauf: Linie ziehen.
      // Nach Voreinstellung wird nur der letzte Stift am Ende
      // der Federkette abesenkt. Sollen alle Stifte gesenkt
      // (d.h. mehr Kurven gezeichnet werden), ist die Bedingung
      // "i==n-1" zu entfernen.
      if(!firstp && i==n-1)
      {
        myline((int)x0[i], (int)y0[i], (int)x1[i], (int)y1[i], cga[c], g);
      }
      
      // Merken der Koordinaten in neuen Speicherplätzen für den nächsten Durchlauf
      x1[i] = x0[i];
      y1[i] = y0[i];
    }
    
    firstp = false;
    
    // Verändern der Parameter
    for(int i = 0; i<n; i++)
    {
      Liss[i].Ax *= Axm; //1.00001;
      Liss[i].Ay *= Aym; //1.00001;
      Liss[i].fy += fp;
      Liss[i].delta += dp;
    }
  }
}